/*
 * Copyright (c) 2009 EBM Websourcing, http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.registry.api.util;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.Properties;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author Christophe HAMERLING - eBM WebSourcing
 * 
 */
public class XMLUtil {

    private static final Log logger = LogFactory.getLog(XMLUtil.class);

    private final static ThreadLocal<DocumentBuilder> documentBuilderThreadLocal = new ThreadLocal<DocumentBuilder>() {

        @Override
        protected DocumentBuilder initialValue() {
            try {
                final DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory
                        .newInstance();
                documentBuilderFactory.setNamespaceAware(true);
                final DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
                return documentBuilder;
            } catch (ParserConfigurationException e) {
                throw new RuntimeException("Failed to create DocumentBuilder", e);
            }
        }
    };

    /**
     * <p>
     * This transformer has the following features:
     * <ul>
     * <li>omit the XML declaration,</li>
     * <li>default encoding</li>
     * </ul>
     * </p>
     */
    private final static ThreadLocal<Transformer> transformerWithoutXmlDeclarationThreadLocal = new ThreadLocal<Transformer>() {

        @Override
        protected Transformer initialValue() {
            try {

                final Transformer transformer = TransformerFactory.newInstance().newTransformer();
                Properties props = new Properties();
                props.put(OutputKeys.OMIT_XML_DECLARATION, "yes");
                props.put(OutputKeys.METHOD, "xml");
                transformer.setOutputProperties(props);
                return transformer;

            } catch (TransformerConfigurationException e) {
                throw new RuntimeException("Failed to create Transformer", e);
            }
        }
    };

    /**
     * <p>
     * This transformer has the following features:
     * <ul>
     * <li>doesn't omit the XML declaration,</li>
     * <li>default encoding</li>
     * </ul>
     * </p>
     */
    private final static ThreadLocal<Transformer> defaultTransformerThreadLocal = new ThreadLocal<Transformer>() {

        @Override
        protected Transformer initialValue() {
            try {

                final Transformer transformer = TransformerFactory.newInstance().newTransformer();
                Properties props = new Properties();
                props.put(OutputKeys.OMIT_XML_DECLARATION, "no");
                props.put(OutputKeys.METHOD, "xml");
                transformer.setOutputProperties(props);
                return transformer;

            } catch (TransformerConfigurationException e) {
                throw new RuntimeException("Failed to create Transformer", e);
            }
        }
    };

    public static Document createDocumentFromString(final String xml) {
        if (xml!=null){
            Document doc = null;
            try {
                InputStream in = new ByteArrayInputStream(xml.getBytes("UTF-8"));
                InputSource inputSource = new InputSource(in);
                doc = XMLUtil.documentBuilderThreadLocal.get().parse(inputSource);
                doc.normalize();
            } catch (SAXException e) {
                logger.error("Bad XML fragment can't be transformed to a DOM tree.", e);
            } catch (IOException e) {
                logger.error("Unexpected Error", e);
            }
    
            return doc;
        } else {
            return null;
        }
    }

    public static String createStringFromDOMDocument(final Node document)
            throws TransformerException {
        return createStringFromDOMNode(document, false);
    }

    public static String createStringFromDOMNode(final Node node) throws TransformerException {
        return createStringFromDOMNode(node, true);
    }

    public static String createStringFromDOMNode(final Node node, final boolean omitDeclaration)
            throws TransformerException {
        node.normalize();
        final Source source = new DOMSource(node);
        final StringWriter out = new StringWriter();
        final Result resultStream = new StreamResult(out);
        if (omitDeclaration) {
            XMLUtil.transformerWithoutXmlDeclarationThreadLocal.get().transform(source,
                    resultStream);
        } else {
            XMLUtil.defaultTransformerThreadLocal.get().transform(source, resultStream);
        }
        return out.toString();
    }

}
